@@ -718,8 +718,16 @@ ldns_key_rr2ds(const ldns_rr *key, ldns_hash h)
 			}
 		break;
 		case LDNS_SHA256:
+#ifdef SHA256_DIGEST_LENGTH
+			digest = LDNS_XMALLOC(uint8_t, SHA256_DIGEST_LENGTH);
+			if (!digest) {
+				ldns_rr_free(ds);
+				return NULL;
+			}
+#else
 			ldns_rr_free(ds);
-			return NULL; /* not implemented */
+			return NULL;
+#endif
 		break;
 	}
 
@@ -745,12 +753,16 @@ ldns_key_rr2ds(const ldns_rr *key, ldns_hash h)
 
         /* digest */
         /* owner name */
-	if (ldns_rdf2buffer_wire(data_buf, ldns_rr_owner(key)) != LDNS_STATUS_OK) {
+        tmp = ldns_rdf_clone(ldns_rr_owner(key));
+        ldns_dname2canonical(tmp);
+	if (ldns_rdf2buffer_wire(data_buf, tmp) != LDNS_STATUS_OK) {
 		LDNS_FREE(digest);
 		ldns_buffer_free(data_buf);
 		ldns_rr_free(ds);
+		ldns_rdf_deep_free(tmp);
 		return NULL;
 	}
+	ldns_rdf_deep_free(tmp);
 
         /* all the rdata's */
 	if (ldns_rr_rdata2buffer_wire(data_buf, (ldns_rr*)key) != LDNS_STATUS_OK) { 
@@ -771,6 +783,14 @@ ldns_key_rr2ds(const ldns_rr *key, ldns_hash h)
 
 		break;
 		case LDNS_SHA256:
+#ifdef SHA256_DIGEST_LENGTH
+		(void) SHA256((unsigned char *) ldns_buffer_begin(data_buf),
+		            ldns_buffer_position(data_buf),
+		            (unsigned char*) digest);
+		tmp = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_HEX, SHA256_DIGEST_LENGTH,
+				digest);
+		ldns_rr_push_rdf(ds, tmp);
+#endif
 		break;
 	}
 
