@@ -1138,7 +1138,9 @@ ldns_rr_clone(const ldns_rr *rr)
 	if (!new_rr) {
 		return NULL;
 	}
-	ldns_rr_set_owner(new_rr, ldns_rdf_clone(ldns_rr_owner(rr)));
+	if (ldns_rr_owner(rr)) {
+		ldns_rr_set_owner(new_rr, ldns_rdf_clone(ldns_rr_owner(rr)));
+  	}
 	ldns_rr_set_ttl(new_rr, ldns_rr_ttl(rr));
 	ldns_rr_set_type(new_rr, ldns_rr_get_type(rr));
 	ldns_rr_set_class(new_rr, ldns_rr_get_class(rr));
@@ -1335,7 +1337,9 @@ int ldns_rr_compare_wire(ldns_buffer *rr1_buf, ldns_buffer *rr2_buf)
         while (offset < rr1_len && *ldns_buffer_at(rr1_buf, offset) != 0) {
           offset += *ldns_buffer_at(rr1_buf, offset) + 1;
         }
-        offset += 9;
+        /* jump to rdata section (PAST the rdata length field, otherwise
+           rrs with different lengths might be sorted erroneously */
+        offset += 11;
 	min_len = (rr1_len < rr2_len) ? rr1_len : rr2_len;
         /* Compare RRs RDATA byte for byte. */
         for(i = offset; i < min_len; i++) {
